<?php
abstract class PEAR_Server_Exception extends Exception
{
    protected $_data = array();
    /**
     * @param string
     * @param array
     */
    function __construct($msg)
    {
        foreach ($this->_data as $name => $value)
        {
            $msg = str_replace("%$name%", $value, $msg);
        }
        parent::__construct($msg);
    }

    public function getData()
    {
        return $this->_data;
    }
}

class PEAR_Server_ExceptionPackageExists extends PEAR_Server_Exception
{
    protected $_message = 'Package "%package%" in Channel "%channel%" already exists';
    public function __construct($package, $channel, $msg = false)
    {
        $this->_data = array('package' => $package, 'channel' => $channel);
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionPackageDoesntExist extends PEAR_Server_Exception
{
    protected $_message = 'Package "%package%" in Channel "%channel%" does not exist';
    public function __construct($package, $channel, $msg = false)
    {
        $this->_data = array('package' => $package, 'channel' => $channel);
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionParentPackageDoesntExist extends PEAR_Server_ExceptionPackageDoesntExist
{
    protected $_message = 'Parent package "%package%" in Channel "%channel%" does not exist';
}

class PEAR_Server_ExceptionMaintainerExists extends PEAR_Server_Exception
{
    protected $_message = 'Maintainer "%handle%" already exists, cannot add';
    public function __construct($handle, $msg = false)
    {
        $this->_data = array('handle' => $handle);
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionMaintainerDoesntExist extends PEAR_Server_ExceptionMaintainerExists
{
    protected $_message = 'Maintainer "%handle%" does not exist, cannot update';
}

class PEAR_Server_ExceptionPackageMaintainerDoesntExist extends PEAR_Server_Exception
{
    protected $_message = 'Package "%package%" in channel "%channel%" is not maintained by "%maintainer%"';
    public function __construct($package, $channel, $maintainer, $msg = false)
    {
        $this->_data = array('package' => $package, 'channel' => $channel, 'maintainer' => $maintainer);
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionPackageMaintainerExists extends PEAR_Server_ExceptionPackageMaintainerDoesntExist
{
    protected $_message = 'Package "%package%" in channel "%channel%" is already maintained by "%maintainer%", use update instead of add';
}

class PEAR_Server_ExceptionNoXmlrpc extends PEAR_Server_Exception
{
    protected $_message = 'XML-RPC extension must be loaded to use the xml-rpc frontend';
    public function __construct($msg = false)
    {
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionXmlrpcUnknownMethod extends PEAR_Server_Exception
{
    protected $_message = 'Unknown XML-RPC method "%method%" requested';
    public function __construct($method, $msg = false)
    {
        $this->_data = array('method' => $method);
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionXmlrpcUnknownSignature extends PEAR_Server_Exception
{
    protected $_message = 'Invalid Method signature "%signature%" for XML-RPC method "%method%"';
    public function __construct($method, $signature, $msg = false)
    {
        $this->_data = array('method' => $method, 'signature' => $signature);
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionNeedPayload extends PEAR_Server_Exception
{
    protected $_message = 'This page may not be accessed directly, but only through xml-rpc';
    public function __construct($msg = false)
    {
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionChannelSetup extends PEAR_Server_Exception
{
    protected $_message = 'Channel "%channel%" must be manually set up in the database prior to use';
    public function __construct($channel, $msg = false)
    {
        $this->_data = array('channel' => $channel);
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionDoesntMaintain extends PEAR_Server_Exception
{
    protected $_message = 'Maintainer "%maintainer%" does not maintain "%channel%::%package%", and cannot release a new version';
    public function __construct($maintainer, $package, $channel, $msg = false)
    {
        $this->_data = array('channel' => $channel, 'package' => $package, 'maintainer' => $maintainer);
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionMaintainerNotLead extends PEAR_Server_ExceptionDoesntMaintain
{
    protected $_message = 'Maintainer "%maintainer%" is not a lead maintainer of "%channel%::%package%", and cannot release a new version';
}

class PEAR_Server_ExceptionChannelNotLocal extends PEAR_Server_Exception
{
    protected $_message = 'Channel "%channel%" is not a local channel (is it in the database?)';
    public function __construct($channel, $msg = false)
    {
        $this->_data = array('channel' => $channel);
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionNeedChannel extends PEAR_Server_Exception
{
    protected $_message = 'getDownloadURL expects index "channel" and index "package" to be set in the first parameter';

    public function __construct($msg = false)
    {
        $this->_data = array();
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionInvalidState extends PEAR_Server_Exception
{
    protected $_message = 'getDownloadURL: preferred state "%state%" is not a valid stability state';
    public function __construct($state, $msg = false)
    {
        $this->_data = array('state' => $state);
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionGroupNotFound extends PEAR_Server_Exception
{
    protected $_message = 'getDownloadURL: package "%package%" version "%version%" has no optional dependency groups';

    public function __construct($package, $version, $msg = false)
    {
        $this->_data = array('package' => $package, 'version' => $version);
        parent::__construct($msg ? $msg : $this->_message);
    }
}

class PEAR_Server_ExceptionXsdversion1_0 extends PEAR_Server_Exception
{
    protected $_message = 'getDepDownloadURL: package.xml dependencies cannot depend on channels other than pear.php.net';

    public function __construct($msg = false)
    {
        $this->_data = array();
        parent::__construct($msg ? $msg : $this->_message);
    }
}
?>