<?php
require_once 'PEAR/Server/Frontend.php';
require_once 'HTML/QuickForm.php';
class PEAR_Server_Frontend_HTMLQuickForm extends PEAR_Server_Frontend
{

    /**
     * @var HTML_QuickForm
     */
    protected $_quickForm;
    protected $_index;
    protected $_tmpdir;
    protected $_user = false;
    protected $_admin = false;
    public function __construct($channel, $qf, $index, $dir)
    {
        parent::PEAR_Server_Frontend($channel);
        $this->_quickForm = $qf;
        $this->_index = $index;
        $this->_tmpdir = $dir;
    }

    public function sessionStart()
    {
        session_start();
        if (session_is_registered('_currentUser')) {
            $this->_user = $_SESSION['_currentUser'];
            $this->_admin = $_SESSION['_currentUserAdmin'];
        } else {
            $this->_user = false;
            $this->_admin = false;
            session_register('_currentUser');
            session_register('_currentUserAdmin');
            $_SESSION['_currentUser'] = false;
            $_SESSION['_currentUserAdmin'] = false;
        }
    }

    public function setServer($server)
    {
        parent::setServer($server);
        $this->_server->addMethodIndex('managePackage');
        $this->_server->addMethodIndex('manageMaintainer');
        $this->_server->addMethodIndex('addMaintainer');
        $this->_server->addMethodIndex('managePackageMaintainers');
        $this->_server->addMethodIndex('login');
    }

    /**
     * Return output to the user from a function
     * @param mixed
     */
    public function funcReturn($output, $class, $method)
    {
        return $output;
    }

    public function main()
    {
        if (isset($_REQUEST['logout'])) {
            $this->_user = false;
            session_destroy();
        }
        if (!$this->_user) {
            return $this->doLogin();
        }
        $func = $this->_server->getMethod(@$_REQUEST['f']);
        if ($func !== false) {
            $this->parseInput($func);
        } else {
            if (isset($_REQUEST['f'])) {
                //error
            } else {
                return $this->doMenu();
            }
        }
    }

    public function parseInput($func)
    {
        switch ($func) {
            case 'managePackage' :
                $this->doMainMenu('doManagePackage', $func, $_REQUEST['managepackage']);
            break;
            case 'managePackageMaintainers' :
                $this->doMainMenu('doManagePackageMaintainers', $func, $_REQUEST['managepackage']);
            break;
            case 'manageMaintainer' :
                $this->doMainMenu('doManageMaintainer', $func, $_REQUEST['managemaintainer']);
            break;
            case 'addMaintainer' :
                $this->doMainMenu('doAddMaintainer', $func);
            break;
            case 'addRelease' :
                $this->doMainMenu('doAddRelease', $func);
            break;
            case 'deleteRelease' :
                $this->doMainMenu('doDeleteRelease', $func);
            break;
            case 'getDownloadURL' :
            case 'addPackage' :
                $this->doMainMenu('doAddPackage', $func);
            break;
            case 'listPackages' :
            case 'listReleases' :
            case 'listLatestReleases' :
            case 'packageInfo' :
            default:
                return $this->doMenu();
            break;
        }
    }

    public function doLogin()
    {
        $this->_quickForm->addElement('header', '', 'Log in');
        $this->_quickForm->addElement('text', 'user', 'User Name');
        $this->_quickForm->addElement('password', 'password', 'Password');
        $this->_quickForm->addElement('submit', 'login', 'Submit');
        $this->_quickForm->addRule('user', 'Required', 'required');
        $this->_quickForm->addRule('password', 'Required', 'required');
        if (isset($_REQUEST['login'])) {
            if ($this->_quickForm->validate()) {
                if ($this->_backend->validLogin(
                      trim(strtolower($this->_quickForm->getSubmitValue('user'))),
                      trim($this->_quickForm->getSubmitValue('password')))) {
                    $_SESSION['_currentUser'] = trim(strtolower($this->_quickForm->getSubmitValue('user')));
                    session_write_close();
                    $this->_user = trim(strtolower($this->_quickForm->getSubmitValue('user')));
                    if ($this->_backend->isAdmin($this->_user)) {
                        $this->_admin = true;
                    } else {
                        $this->_admin = false;
                    }
                    $_SESSION['_currentUserAdmin'] = $this->_admin;
                    return $this->doMenu();
                } else {
                    echo "Invalid Login<br />";
                }
            }
        }
        echo $this->_quickForm->toHtml();
    }

    public function doMenu()
    {
        return $this->doMainMenu('dummy', false);
    }
    
    private function dummy(){}
    
    public function doMainMenu($menu, $func, $param = null)
    {?>
<html>
<head>
<title>Channel <?php echo $this->_channel; ?> Server Administration</title>
</head>
<body>Logged in as <strong><?php echo $this->_user; ?></strong> <a href="<?php echo $this->_index; ?>?logout=1">Logout</a><br />
<?php $this->$menu($func, $param); $this->adminMenu(); ?>
</body>
</html>
<?php
    }

    public function userMenu()
    {?>
<a href="<?php echo $this->_index; ?>?f=<?php echo $this->_server->getMethodIndex('addRelease'); ?>">Upload a Release</a><br />
Manage Existing Packages:<br />
<blockquote>
<?php
foreach ($this->_backend->listPackages(false, false, false) as $package) {
    if (!$this->_backend->packageLead($package['package'], $this->_user)) {
        continue;
    }
    echo '<a href="' . $this->_index . '?f=' . $this->_server->getMethodIndex('managePackage') . '&managepackage=' . $package['package'] . '">'
        . $package['package'] . "</a><br />\n";
}    
?>
</blockquote>
<?php
    }

    public function adminMenu()
    {
        if (!$this->_backend->isAdmin($this->_user)) {
            return $this->userMenu();
        }
        ?>
<a href="<?php echo $this->_index; ?>?f=<?php echo $this->_server->getMethodIndex('addPackage'); ?>">Create a Package</a><br />
<a href="<?php echo $this->_index; ?>?f=<?php echo $this->_server->getMethodIndex('addRelease'); ?>">Upload a Release</a><br />
<a href="<?php echo $this->_index; ?>?f=<?php echo $this->_server->getMethodIndex('addMaintainer'); ?>">Add a Maintainer</a><br />
Manage Existing Packages:<br />
<blockquote>
<?php
foreach ($this->_backend->listPackages(false, false, false) as $package) {
    echo '<a href="' . $this->_index . '?f=' . $this->_server->getMethodIndex('managePackage') . '&managepackage=' . $package['package'] . '">'
        . $package['package'] . "</a><br />\n";
}    
?>
</blockquote>

Manage Existing Maintainers:<br />
<blockquote>
<?php
foreach ($this->_backend->listMaintainers() as $maintainer) {
    echo '<a href="' . $this->_index . '?f=' . $this->_server->getMethodIndex('manageMaintainer') . '&managemaintainer=' . $maintainer->handle . '">'
        . $maintainer->handle . ' (' . $maintainer->name . ')' . "</a><br />\n";
}    
?>
</blockquote>
<?php
    }

    public function doAddRelease($func)
    {
        $this->_quickForm->setDefaults(array(
            'submitted' => 1,
            'f' => $_REQUEST['f'],
        ));
        $this->_quickForm->setConstants(array('f' => $this->_server->getMethodIndex($func)));
        $this->_quickForm->addElement('header', '', 'Upload a Package Release');
        $this->_quickForm->addElement('file', 'release', '.tgz release');
        $this->_quickForm->addElement('hidden', 'submitted', '1');
        $this->_quickForm->addElement('hidden', 'f', 'f');
        $this->_quickForm->addElement('submit', 'Submit', 'Submit');
        $this->_quickForm->addRule('release', 'Required', 'required');
        $this->_quickForm->addRule('release', 'Must be a valid file on your computer', 'uploadedfile');
        if (isset($_REQUEST['submitted']) && $this->_quickForm->validate()) {
            $file = $this->_quickForm->getElement('release');
            if ($file->isUploadedFile()) {
                $fullpath = str_replace(array('/', '\\'), array(DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR),
                    tempnam($this->_tmpdir, 'upl'));
                $file->moveUploadedFile($this->_tmpdir, basename($fullpath));
                $release = new PEAR_Server_Release($fullpath, $this->_user, PEAR_Config::singleton(),
                    $this->_tmpdir);
                try {
                    if ($this->_server->addRelease($release)) {
                        echo 'Release successfully saved<br />';
                    } else {
                        echo '<strong>Error:</strong> Saving release failed<br />';
                        echo $this->_quickForm->toHtml();
                    }
                } catch (PEAR_Server_Exception $e) {
                    echo '<strong>Error:</strong> ' . $e->getMessage() . '<br />';
                    echo $this->_quickForm->toHtml();
                }
            } else {
                echo '<strong>Error:</strong> Not uploaded file<br />';
                echo $this->_quickForm->toHtml();
            }
            return;
        } else {
            echo $this->_quickForm->toHtml();
        }
    }

    public function doManagePackage($fn, $package)
    {
        try {
            if (isset($_REQUEST['deleteRelease'])) {
                if ($this->_server->deleteRelease($this->_channel, $package, $_REQUEST['deleteRelease']));
            }
            $self = '<a href="' . $this->_index . '?f=' .
                $this->_server->getMethodIndex('managePackage') . '&managepackage=' . $package;
            $info = $this->_backend->getPackage($package);
            if (isset($_REQUEST['submitted'])) {
                $stuff = $this->_quickForm->getSubmitValues();
                foreach ($stuff as $name => $value) {
                    $info->$name = $value;
                }
                $this->_backend->updatePackage($info);
                $this->_quickForm->addElement('header', '', 'Data Saved');
            }
            $defaults = $info->toArray();
            $defaults['managepackage'] = $info->name;
            $defaults['f'] = $_REQUEST['f'];
            $this->_quickForm->setDefaults($defaults);
            $this->_quickForm->addElement('header', '', 'Edit Package Information');
            $this->_quickForm->addElement('html', '<tr><td colspan="2"><a href="' . $this->_index . '?f=' .
                $this->_server->getMethodIndex('managePackageMaintainers') . '&managepackage=' . $package .
                '">Manage Package Maintainers</a></td></tr>');
            $this->_quickForm->addElement('static', 'channel', 'Channel');
            $this->_quickForm->addElement('text', 'name', 'Name');
            $this->_quickForm->addElement('text', 'license', 'License');
            $this->_quickForm->addElement('hidden', 'f', 'License');
            $this->_quickForm->addElement('hidden', 'managepackage', 'License');
            $this->_quickForm->addElement('text', 'summary', 'Summary', array('size' => 80));
            $this->_quickForm->addElement('textarea', 'description', 'Description', array('cols' => 80, 'rows' => 5));
            $p = $this->_backend->listPackages(false, false, false);
            $packages = array('');
            foreach ($p as $pack) {
                if ($package['package'] == $pack) {
                    continue;
                }
                $packages[] = $pack['package'];
            }
            $this->_quickForm->addElement('select', 'parent', 'Parent Package', $packages);
            $this->_quickForm->getElement('parent')->setSelected($info->parent);
            $this->_quickForm->addElement('submit', 'submitted', 'Save Changes');
            $this->_quickForm->addRule('name', 'Required', 'required');
            $this->_quickForm->addRule('license', 'Required', 'required');
            $this->_quickForm->addRule('summary', 'Required', 'required');
            $this->_quickForm->addRule('description', 'Required', 'required');
            echo $this->_quickForm->toHtml();
            $releases = $this->_backend->listReleases($package);
            if (count($releases)) {
                echo "Delete Existing Releases:<br />\n";
                array_reverse($releases);
                foreach ($releases as $version => $info) {
                    echo $self . '&deleteRelease=' . urlencode($version) . '">X</a> Version ' .
                         $version . ', Released on ' . $info['releasedate'] . ' by ' . $info['maintainer'] . "<br />\n";
                }
            }
        } catch (PEAR_Server_Exception $e) {
            echo "<strong>Error</strong> " . $e->getMessage();
        }
    }

    public function doManagePackageMaintainers($fn, $package)
    {
        try {
            $self = '<a href="' . $this->_index . '?f=' .
                $this->_server->getMethodIndex('managePackageMaintainers') . '&managepackage=' . $package;
            $this->_quickForm->addElement('static', 'channel', 'Channel');
            $this->_quickForm->addElement('static', 'package', 'Package Name');
            $this->_quickForm->addElement('header', '', 'Add Maintainer To Package');
            $oldroles = PEAR_Common::getUserRoles();
            $roles = array();
            foreach ($oldroles as $role) {
                $roles[$role] = $role;
            }
            $oldmaints = $this->_backend->listMaintainers(true);
            $maints = array('' => '');
            foreach ($oldmaints as $om) {
                $maints[$om] = $om;
            }
            $this->_quickForm->addElement('select', 'newhandle', 'New Maintainer', $maints);
            array_unshift($roles, '');
            $this->_quickForm->addElement('select', 'newrole', 'Role', $roles);
            $this->_quickForm->addElement('select', 'newactive', 'Active Maintainer?', array('' => '', 0 => 0, 1=> 1));
            $this->_quickForm->addElement('submit', 'addmaintainer', 'Add Maintainer');
            $this->_quickForm->addFormRule(array($this, 'checkEmpty'));
            if (isset($_REQUEST['submitted'])) {
                if ($this->_quickForm->getSubmitValue('savechanges') == 'Save Changes') {
                    $submitted = $this->_quickForm->getSubmitValues();
                    foreach ($this->_backend->listPackageMaintainers($package) as $maintainer) {
                        $groupname = 'group' . $maintainer->handle;
                        $values = $submitted[$groupname];
                        $info = new PEAR_Server_MaintainerPackage($values);
                        $info->handle = $maintainer->handle;
                        $info->channel = $this->_channel;
                        $info->package = $package;
                        if ($this->_backend->updatePackageMaintainer($info)) {
                            $this->_quickForm->addElement('header', '', 'Changes to ' . $info->handle . ' Saved');
                        } else {
                            $this->_quickForm->addElement('header', '', 'Changes to ' . $info->handle . ' Failed!');
                        }
                    }
                } elseif ($this->_quickForm->getSubmitValue('addmaintainer') == 'Add Maintainer') {
                    if ($this->_quickForm->validate()) {
                        $info = new PEAR_Server_MaintainerPackage;
                        $info->channel = $this->_channel;
                        $info->package = $package;
                        $info->handle = $this->_quickForm->getSubmitValue('newhandle');
                        $info->role = $this->_quickForm->getSubmitValue('newrole');
                        $info->active = $this->_quickForm->getSubmitValue('newactive');
                        try {
                            if ($this->_backend->addPackageMaintainer($info)) {
                                $this->_quickForm->addElement('header', '', 'Addition of ' . $info->handle . ' to Package Succeeded');
                            } else {
                                $this->_quickForm->addElement('header', '', 'Addition of ' . $info->handle . ' Failed!');
                            }
                        } catch (PEAR_Server_Exception $e) {
                            echo "<strong>Error</strong> " . $e->getMessage() . '<br />';
                        }
                    }
                }
            }
            $defaults = array();
            $defaults['channel'] = $this->_channel;
            $defaults['submitted'] = 1;
            $defaults['package'] = $package;
            $defaults['managepackage'] = $package;
            $defaults['f'] = $_REQUEST['f'];
            $this->_quickForm->setDefaults($defaults);
            $this->_quickForm->addElement('header', '', 'Edit Package Maintainers');
            $this->_quickForm->addElement('hidden', 'submitted', '');
            $this->_quickForm->addElement('hidden', 'f', 'License');
            $this->_quickForm->addElement('hidden', 'managepackage', 'License');
            $this->_quickForm->addElement('header', '', '<div align="right">Role | Active</div>');
            $genericrole = HTML_QuickForm::createElement('select', 'role', '', $roles);
            $genericactive = HTML_QuickForm::createElement('select', 'active', '', array(0, 1));
            foreach ($this->_backend->listPackageMaintainers($package) as $maintainer) {
                $role = clone $genericrole;
                $active = clone $genericactive;
                $role->setSelected($maintainer->role);
                $active->setSelected($maintainer->active);
                $this->_quickForm->addGroup(array($role, $active), 'group' . $maintainer->handle, $maintainer->handle, '|');
            }
            $this->_quickForm->addElement('submit', 'savechanges', 'Save Changes');
            echo $this->_quickForm->toHtml();
        } catch (PEAR_Server_Exception $e) {
            echo "<strong>Error</strong> " . $e->getMessage();
        }
    }

    public function checkEmpty($handle)
    {
        if ($_REQUEST['addmaintainer'] == 'Add Maintainer') {
            $ret = array();
            if (trim($handle['newhandle']) == '' ||
                trim($handle['newactive'] == '') ||
                trim($handle['newrole'] == '')) {
                if (trim($handle['newhandle']) == '') {
                    $ret['newhandle'] = 'Required';
                }
                if (trim($handle['newactive']) == '') {
                    $ret['newactive'] = 'Required';
                }
                if (trim($handle['newrole']) == '') {
                    $ret['newrole'] = 'Required';
                }
                return $ret;
            }
        }
        return true;
    }

    public function doManageMaintainer($fn, $maintainer)
    {
        try {
            $info = $this->_backend->getMaintainer($maintainer);
            if (isset($_REQUEST['submitted'])) {
                if ($this->_quickForm->validate()) {
                    $stuff = $this->_quickForm->getSubmitValues();
                    foreach ($stuff as $name => $value) {
                        $info->$name = $value;
                    }
                    $this->_backend->updateMaintainer($info);
                    $this->_quickForm->addElement('header', '', 'Data Saved');
                }
            }
            $defaults = $info->toArray();
            $defaults['managemaintainer'] = $info->handle;
            $defaults['f'] = $_REQUEST['f'];
            $this->_quickForm->setDefaults($defaults);
            $this->_quickForm->addElement('header', '', 'Edit Maintainer Information');
            $this->_quickForm->addElement('static', 'handle', 'Handle');
            $this->_quickForm->addElement('text', 'name', 'Name', array('size' => 80));
            $this->_quickForm->addElement('hidden', 'f', 'License');
            $this->_quickForm->addElement('hidden', 'managemaintainer', 'License');
            $this->_quickForm->addElement('text', 'email', 'Email', array('size' => 80));
            $this->_quickForm->addElement('submit', 'submitted', 'Save Changes');
            $this->_quickForm->addRule('name', 'Required', 'required');
            $this->_quickForm->addRule('email', 'Required', 'required');
            echo $this->_quickForm->toHtml();
        } catch (PEAR_Server_Exception $e) {
            echo "<strong>Error</strong> " . $e->getMessage();
        }
    }

    public function doAddPackage($func)
    {
        $defaults = array();
        $defaults['f'] = $_REQUEST['f'];
        $defaults['submitted'] = 1;
        $defaults['channel'] = $this->_channel;
        $this->_quickForm->setDefaults($defaults);
        $this->_quickForm->addElement('header', '', 'Add a New Package');
        $this->_quickForm->addElement('static', 'channel', 'Channel');
        $this->_quickForm->addElement('text', 'name', 'Name');
        $this->_quickForm->addElement('text', 'license', 'License');
        $this->_quickForm->addElement('hidden', 'f', 'License');
        $this->_quickForm->addElement('hidden', 'submitted', '1');
        $this->_quickForm->addElement('text', 'summary', 'Summary', array('size' => 80));
        $this->_quickForm->addElement('textarea', 'description', 'Description', array('cols' => 80, 'rows' => 5));
        $p = $this->_backend->listPackages(false, false, false);
        $packages = array('');
        foreach ($p as $package) {
            $packages[] = $package['package'];
        }
        $this->_quickForm->addElement('select', 'parent', 'Parent Package', $packages);
        $this->_quickForm->addElement('submit', 'save', 'Save Changes');
        $this->_quickForm->addRule('name', 'Required', 'required');
        $this->_quickForm->addRule('license', 'Required', 'required');
        $this->_quickForm->addRule('summary', 'Required', 'required');
        $this->_quickForm->addRule('description', 'Required', 'required');
        if (isset($_REQUEST['submitted'])) {
            if ($this->_quickForm->validate()) {
                try {
                    $spack = new PEAR_Server_Package;
                    $spack->name = trim($this->_quickForm->getSubmitValue('name'));
                    $spack->channel = $this->_channel;
                    $spack->license = trim($this->_quickForm->getSubmitValue('license'));
                    $spack->summary = trim($this->_quickForm->getSubmitValue('summary'));
                    $spack->description = trim($this->_quickForm->getSubmitValue('description'));
                    if ($this->_backend->addPackage($spack)) {
                        $this->_quickForm->removeElement('save');
                        $this->_quickForm->freeze();
                        echo '<strong>Package Successfully Added</strong>';
                    } else {
                        echo '<strong>Warning: adding package failed';
                    }
                } catch (PEAR_Server_Exception $e) {
                    echo "<strong>Error</strong> " . $e->getMessage();
                }
            }
        }
        echo $this->_quickForm->toHtml();
    }

    public function doAddMaintainer($func)
    {
        $defaults = array();
        $defaults['f'] = $_REQUEST['f'];
        $defaults['submitted'] = 1;
        $defaults['channel'] = $this->_channel;
        $this->_quickForm->setDefaults($defaults);
        $this->_quickForm->addElement('header', '', 'Add a New Maintainer');
        $this->_quickForm->addElement('text', 'handle', 'Handle');
        $this->_quickForm->addElement('text', 'name', 'Name');
        $this->_quickForm->addElement('text', 'email', 'Email');
        $this->_quickForm->addElement('hidden', 'f', 'License');
        $this->_quickForm->addElement('hidden', 'submitted', '1');
        $this->_quickForm->addElement('submit', 'save', 'Save Changes');
        $this->_quickForm->addRule('name', 'Required', 'required');
        $this->_quickForm->addRule('handle', 'Required', 'required');
        $this->_quickForm->addRule('email', 'Required', 'required');
        $this->_quickForm->addRule('handle', 'Maximum length 20 characters', 'maxlength', 20);
        $this->_quickForm->addRule('name', 'Maximum length 255 characters', 'maxlength', 255);
        $this->_quickForm->addRule('email', 'Maximum length 255 characters', 'maxlength', 255);
        $this->_quickForm->addRule('email', 'Invalid Email Address', 'email');
        if (isset($_REQUEST['submitted'])) {
            if ($this->_quickForm->validate()) {
                try {
                    $spack = new PEAR_Server_Maintainer;
                    $spack->handle = trim($this->_quickForm->getSubmitValue('handle'));
                    $spack->name = trim($this->_quickForm->getSubmitValue('name'));
                    $spack->email = trim($this->_quickForm->getSubmitValue('email'));
                    if ($this->_backend->addMaintainer($spack)) {
                        $this->_quickForm->removeElement('save');
                        $this->_quickForm->freeze();
                        echo '<strong>Maintainer Successfully Added</strong>';
                    } else {
                        echo '<strong>Warning: adding maintainer failed';
                    }
                } catch (PEAR_Server_Exception $e) {
                    echo "<strong>Error</strong> " . $e->getMessage();
                }
            }
        }
        echo $this->_quickForm->toHtml();
    }

    public function getProtocols()
    {
        return array();
    }

    public function getOutput()
    {
    }
}
?>